# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..core.request_options import RequestOptions
from ..core.http_response import HttpResponse
from ..types.batch_delete_catalog_objects_response import BatchDeleteCatalogObjectsResponse
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.batch_get_catalog_objects_response import BatchGetCatalogObjectsResponse
from ..requests.catalog_object_batch import CatalogObjectBatchParams
from ..types.batch_upsert_catalog_objects_response import BatchUpsertCatalogObjectsResponse
from ..core.serialization import convert_and_respect_annotation_metadata
from ..types.catalog_info_response import CatalogInfoResponse
from ..types.catalog_object_type import CatalogObjectType
from ..requests.catalog_query import CatalogQueryParams
from ..types.search_catalog_objects_response import SearchCatalogObjectsResponse
from ..types.search_catalog_items_request_stock_level import SearchCatalogItemsRequestStockLevel
from ..types.sort_order import SortOrder
from ..types.catalog_item_product_type import CatalogItemProductType
from ..requests.custom_attribute_filter import CustomAttributeFilterParams
from ..types.archived_state import ArchivedState
from ..types.search_catalog_items_response import SearchCatalogItemsResponse
from ..types.update_item_modifier_lists_response import UpdateItemModifierListsResponse
from ..types.update_item_taxes_response import UpdateItemTaxesResponse
from ..core.client_wrapper import AsyncClientWrapper
from ..core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawCatalogClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def batch_delete(
        self, *, object_ids: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[BatchDeleteCatalogObjectsResponse]:
        """
        Deletes a set of [CatalogItem](entity:CatalogItem)s based on the
        provided list of target IDs and returns a set of successfully deleted IDs in
        the response. Deletion is a cascading event such that all children of the
        targeted object are also deleted. For example, deleting a CatalogItem will
        also delete all of its [CatalogItemVariation](entity:CatalogItemVariation)
        children.

        `BatchDeleteCatalogObjects` succeeds even if only a portion of the targeted
        IDs can be deleted. The response will only include IDs that were
        actually deleted.

        To ensure consistency, only one delete request is processed at a time per seller account.
        While one (batch or non-batch) delete request is being processed, other (batched and non-batched)
        delete requests are rejected with the `429` error code.

        Parameters
        ----------
        object_ids : typing.Sequence[str]
            The IDs of the CatalogObjects to be deleted. When an object is deleted, other objects
            in the graph that depend on that object will be deleted as well (for example, deleting a
            CatalogItem will delete its CatalogItemVariation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BatchDeleteCatalogObjectsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/catalog/batch-delete",
            method="POST",
            json={
                "object_ids": object_ids,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BatchDeleteCatalogObjectsResponse,
                    construct_type(
                        type_=BatchDeleteCatalogObjectsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def batch_get(
        self,
        *,
        object_ids: typing.Sequence[str],
        include_related_objects: typing.Optional[bool] = OMIT,
        catalog_version: typing.Optional[int] = OMIT,
        include_deleted_objects: typing.Optional[bool] = OMIT,
        include_category_path_to_root: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[BatchGetCatalogObjectsResponse]:
        """
        Returns a set of objects based on the provided ID.
        Each [CatalogItem](entity:CatalogItem) returned in the set includes all of its
        child information including: all of its
        [CatalogItemVariation](entity:CatalogItemVariation) objects, references to
        its [CatalogModifierList](entity:CatalogModifierList) objects, and the ids of
        any [CatalogTax](entity:CatalogTax) objects that apply to it.

        Parameters
        ----------
        object_ids : typing.Sequence[str]
            The IDs of the CatalogObjects to be retrieved.

        include_related_objects : typing.Optional[bool]
            If `true`, the response will include additional objects that are related to the
            requested objects. Related objects are defined as any objects referenced by ID by the results in the `objects` field
            of the response. These objects are put in the `related_objects` field. Setting this to `true` is
            helpful when the objects are needed for immediate display to a user.
            This process only goes one level deep. Objects referenced by the related objects will not be included. For example,

            if the `objects` field of the response contains a CatalogItem, its associated
            CatalogCategory objects, CatalogTax objects, CatalogImage objects and
            CatalogModifierLists will be returned in the `related_objects` field of the
            response. If the `objects` field of the response contains a CatalogItemVariation,
            its parent CatalogItem will be returned in the `related_objects` field of
            the response.

            Default value: `false`

        catalog_version : typing.Optional[int]
            The specific version of the catalog objects to be included in the response.
            This allows you to retrieve historical versions of objects. The specified version value is matched against
            the [CatalogObject](entity:CatalogObject)s' `version` attribute. If not included, results will
            be from the current version of the catalog.

        include_deleted_objects : typing.Optional[bool]
            Indicates whether to include (`true`) or not (`false`) in the response deleted objects, namely, those with the `is_deleted` attribute set to `true`.

        include_category_path_to_root : typing.Optional[bool]
            Specifies whether or not to include the `path_to_root` list for each returned category instance. The `path_to_root` list consists
            of `CategoryPathToRootNode` objects and specifies the path that starts with the immediate parent category of the returned category
            and ends with its root category. If the returned category is a top-level category, the `path_to_root` list is empty and is not returned
            in the response payload.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BatchGetCatalogObjectsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/catalog/batch-retrieve",
            method="POST",
            json={
                "object_ids": object_ids,
                "include_related_objects": include_related_objects,
                "catalog_version": catalog_version,
                "include_deleted_objects": include_deleted_objects,
                "include_category_path_to_root": include_category_path_to_root,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BatchGetCatalogObjectsResponse,
                    construct_type(
                        type_=BatchGetCatalogObjectsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def batch_upsert(
        self,
        *,
        idempotency_key: str,
        batches: typing.Sequence[CatalogObjectBatchParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[BatchUpsertCatalogObjectsResponse]:
        """
        Creates or updates up to 10,000 target objects based on the provided
        list of objects. The target objects are grouped into batches and each batch is
        inserted/updated in an all-or-nothing manner. If an object within a batch is
        malformed in some way, or violates a database constraint, the entire batch
        containing that item will be disregarded. However, other batches in the same
        request may still succeed. Each batch may contain up to 1,000 objects, and
        batches will be processed in order as long as the total object count for the
        request (items, variations, modifier lists, discounts, and taxes) is no more
        than 10,000.

        To ensure consistency, only one update request is processed at a time per seller account.
        While one (batch or non-batch) update request is being processed, other (batched and non-batched)
        update requests are rejected with the `429` error code.

        Parameters
        ----------
        idempotency_key : str
            A value you specify that uniquely identifies this
            request among all your requests. A common way to create
            a valid idempotency key is to use a Universally unique
            identifier (UUID).

            If you're unsure whether a particular request was successful,
            you can reattempt it with the same idempotency key without
            worrying about creating duplicate objects.

            See [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) for more information.

        batches : typing.Sequence[CatalogObjectBatchParams]
            A batch of CatalogObjects to be inserted/updated atomically.
            The objects within a batch will be inserted in an all-or-nothing fashion, i.e., if an error occurs
            attempting to insert or update an object within a batch, the entire batch will be rejected. However, an error
            in one batch will not affect other batches within the same request.

            For each object, its `updated_at` field is ignored and replaced with a current [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates), and its
            `is_deleted` field must not be set to `true`.

            To modify an existing object, supply its ID. To create a new object, use an ID starting
            with `#`. These IDs may be used to create relationships between an object and attributes of
            other objects that reference it. For example, you can create a CatalogItem with
            ID `#ABC` and a CatalogItemVariation with its `item_id` attribute set to
            `#ABC` in order to associate the CatalogItemVariation with its parent
            CatalogItem.

            Any `#`-prefixed IDs are valid only within a single atomic batch, and will be replaced by server-generated IDs.

            Each batch may contain up to 1,000 objects. The total number of objects across all batches for a single request
            may not exceed 10,000. If either of these limits is violated, an error will be returned and no objects will
            be inserted or updated.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[BatchUpsertCatalogObjectsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/catalog/batch-upsert",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "batches": convert_and_respect_annotation_metadata(
                    object_=batches, annotation=typing.Sequence[CatalogObjectBatchParams], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BatchUpsertCatalogObjectsResponse,
                    construct_type(
                        type_=BatchUpsertCatalogObjectsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def info(self, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[CatalogInfoResponse]:
        """
        Retrieves information about the Square Catalog API, such as batch size
        limits that can be used by the `BatchUpsertCatalogObjects` endpoint.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CatalogInfoResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/catalog/info",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CatalogInfoResponse,
                    construct_type(
                        type_=CatalogInfoResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def search(
        self,
        *,
        cursor: typing.Optional[str] = OMIT,
        object_types: typing.Optional[typing.Sequence[CatalogObjectType]] = OMIT,
        include_deleted_objects: typing.Optional[bool] = OMIT,
        include_related_objects: typing.Optional[bool] = OMIT,
        begin_time: typing.Optional[str] = OMIT,
        query: typing.Optional[CatalogQueryParams] = OMIT,
        limit: typing.Optional[int] = OMIT,
        include_category_path_to_root: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SearchCatalogObjectsResponse]:
        """
        Searches for [CatalogObject](entity:CatalogObject) of any type by matching supported search attribute values,
        excluding custom attribute values on items or item variations, against one or more of the specified query filters.

        This (`SearchCatalogObjects`) endpoint differs from the [SearchCatalogItems](api-endpoint:Catalog-SearchCatalogItems)
        endpoint in the following aspects:

        - `SearchCatalogItems` can only search for items or item variations, whereas `SearchCatalogObjects` can search for any type of catalog objects.
        - `SearchCatalogItems` supports the custom attribute query filters to return items or item variations that contain custom attribute values, where `SearchCatalogObjects` does not.
        - `SearchCatalogItems` does not support the `include_deleted_objects` filter to search for deleted items or item variations, whereas `SearchCatalogObjects` does.
        - The both endpoints have different call conventions, including the query filter formats.

        Parameters
        ----------
        cursor : typing.Optional[str]
            The pagination cursor returned in the previous response. Leave unset for an initial request.
            See [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination) for more information.

        object_types : typing.Optional[typing.Sequence[CatalogObjectType]]
            The desired set of object types to appear in the search results.

            If this is unspecified, the operation returns objects of all the top level types at the version
            of the Square API used to make the request. Object types that are nested onto other object types
            are not included in the defaults.

            At the current API version the default object types are:
            ITEM, CATEGORY, TAX, DISCOUNT, MODIFIER_LIST,
            PRICING_RULE, PRODUCT_SET, TIME_PERIOD, MEASUREMENT_UNIT,
            SUBSCRIPTION_PLAN, ITEM_OPTION, CUSTOM_ATTRIBUTE_DEFINITION, QUICK_AMOUNT_SETTINGS.

            Note that if you wish for the query to return objects belonging to nested types (i.e., COMPONENT, IMAGE,
            ITEM_OPTION_VAL, ITEM_VARIATION, or MODIFIER), you must explicitly include all the types of interest
            in this field.

        include_deleted_objects : typing.Optional[bool]
            If `true`, deleted objects will be included in the results. Defaults to `false`. Deleted objects will have their `is_deleted` field set to `true`. If `include_deleted_objects` is `true`, then the `include_category_path_to_root` request parameter must be `false`. Both properties cannot be `true` at the same time.

        include_related_objects : typing.Optional[bool]
            If `true`, the response will include additional objects that are related to the
            requested objects. Related objects are objects that are referenced by object ID by the objects
            in the response. This is helpful if the objects are being fetched for immediate display to a user.
            This process only goes one level deep. Objects referenced by the related objects will not be included.
            For example:

            If the `objects` field of the response contains a CatalogItem, its associated
            CatalogCategory objects, CatalogTax objects, CatalogImage objects and
            CatalogModifierLists will be returned in the `related_objects` field of the
            response. If the `objects` field of the response contains a CatalogItemVariation,
            its parent CatalogItem will be returned in the `related_objects` field of
            the response.

            Default value: `false`

        begin_time : typing.Optional[str]
            Return objects modified after this [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates), in RFC 3339
            format, e.g., `2016-09-04T23:59:33.123Z`. The timestamp is exclusive - objects with a
            timestamp equal to `begin_time` will not be included in the response.

        query : typing.Optional[CatalogQueryParams]
            A query to be used to filter or sort the results. If no query is specified, the entire catalog will be returned.

        limit : typing.Optional[int]
            A limit on the number of results to be returned in a single page. The limit is advisory -
            the implementation may return more or fewer results. If the supplied limit is negative, zero, or
            is higher than the maximum limit of 1,000, it will be ignored.

        include_category_path_to_root : typing.Optional[bool]
            Specifies whether or not to include the `path_to_root` list for each returned category instance. The `path_to_root` list consists of `CategoryPathToRootNode` objects and specifies the path that starts with the immediate parent category of the returned category and ends with its root category. If the returned category is a top-level category, the `path_to_root` list is empty and is not returned in the response payload. If `include_category_path_to_root` is `true`, then the `include_deleted_objects` request parameter must be `false`. Both properties cannot be `true` at the same time.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SearchCatalogObjectsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/catalog/search",
            method="POST",
            json={
                "cursor": cursor,
                "object_types": object_types,
                "include_deleted_objects": include_deleted_objects,
                "include_related_objects": include_related_objects,
                "begin_time": begin_time,
                "query": convert_and_respect_annotation_metadata(
                    object_=query, annotation=CatalogQueryParams, direction="write"
                ),
                "limit": limit,
                "include_category_path_to_root": include_category_path_to_root,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchCatalogObjectsResponse,
                    construct_type(
                        type_=SearchCatalogObjectsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def search_items(
        self,
        *,
        text_filter: typing.Optional[str] = OMIT,
        category_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        stock_levels: typing.Optional[typing.Sequence[SearchCatalogItemsRequestStockLevel]] = OMIT,
        enabled_location_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        limit: typing.Optional[int] = OMIT,
        sort_order: typing.Optional[SortOrder] = OMIT,
        product_types: typing.Optional[typing.Sequence[CatalogItemProductType]] = OMIT,
        custom_attribute_filters: typing.Optional[typing.Sequence[CustomAttributeFilterParams]] = OMIT,
        archived_state: typing.Optional[ArchivedState] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SearchCatalogItemsResponse]:
        """
        Searches for catalog items or item variations by matching supported search attribute values, including
        custom attribute values, against one or more of the specified query filters.

        This (`SearchCatalogItems`) endpoint differs from the [SearchCatalogObjects](api-endpoint:Catalog-SearchCatalogObjects)
        endpoint in the following aspects:

        - `SearchCatalogItems` can only search for items or item variations, whereas `SearchCatalogObjects` can search for any type of catalog objects.
        - `SearchCatalogItems` supports the custom attribute query filters to return items or item variations that contain custom attribute values, where `SearchCatalogObjects` does not.
        - `SearchCatalogItems` does not support the `include_deleted_objects` filter to search for deleted items or item variations, whereas `SearchCatalogObjects` does.
        - The both endpoints use different call conventions, including the query filter formats.

        Parameters
        ----------
        text_filter : typing.Optional[str]
            The text filter expression to return items or item variations containing specified text in
            the `name`, `description`, or `abbreviation` attribute value of an item, or in
            the `name`, `sku`, or `upc` attribute value of an item variation.

        category_ids : typing.Optional[typing.Sequence[str]]
            The category id query expression to return items containing the specified category IDs.

        stock_levels : typing.Optional[typing.Sequence[SearchCatalogItemsRequestStockLevel]]
            The stock-level query expression to return item variations with the specified stock levels.
            See [SearchCatalogItemsRequestStockLevel](#type-searchcatalogitemsrequeststocklevel) for possible values

        enabled_location_ids : typing.Optional[typing.Sequence[str]]
            The enabled-location query expression to return items and item variations having specified enabled locations.

        cursor : typing.Optional[str]
            The pagination token, returned in the previous response, used to fetch the next batch of pending results.

        limit : typing.Optional[int]
            The maximum number of results to return per page. The default value is 100.

        sort_order : typing.Optional[SortOrder]
            The order to sort the results by item names. The default sort order is ascending (`ASC`).
            See [SortOrder](#type-sortorder) for possible values

        product_types : typing.Optional[typing.Sequence[CatalogItemProductType]]
            The product types query expression to return items or item variations having the specified product types.

        custom_attribute_filters : typing.Optional[typing.Sequence[CustomAttributeFilterParams]]
            The customer-attribute filter to return items or item variations matching the specified
            custom attribute expressions. A maximum number of 10 custom attribute expressions are supported in
            a single call to the [SearchCatalogItems](api-endpoint:Catalog-SearchCatalogItems) endpoint.

        archived_state : typing.Optional[ArchivedState]
            The query filter to return not archived (`ARCHIVED_STATE_NOT_ARCHIVED`), archived (`ARCHIVED_STATE_ARCHIVED`), or either type (`ARCHIVED_STATE_ALL`) of items.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SearchCatalogItemsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/catalog/search-catalog-items",
            method="POST",
            json={
                "text_filter": text_filter,
                "category_ids": category_ids,
                "stock_levels": stock_levels,
                "enabled_location_ids": enabled_location_ids,
                "cursor": cursor,
                "limit": limit,
                "sort_order": sort_order,
                "product_types": product_types,
                "custom_attribute_filters": convert_and_respect_annotation_metadata(
                    object_=custom_attribute_filters,
                    annotation=typing.Sequence[CustomAttributeFilterParams],
                    direction="write",
                ),
                "archived_state": archived_state,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchCatalogItemsResponse,
                    construct_type(
                        type_=SearchCatalogItemsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update_item_modifier_lists(
        self,
        *,
        item_ids: typing.Sequence[str],
        modifier_lists_to_enable: typing.Optional[typing.Sequence[str]] = OMIT,
        modifier_lists_to_disable: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateItemModifierListsResponse]:
        """
        Updates the [CatalogModifierList](entity:CatalogModifierList) objects
        that apply to the targeted [CatalogItem](entity:CatalogItem) without having
        to perform an upsert on the entire item.

        Parameters
        ----------
        item_ids : typing.Sequence[str]
            The IDs of the catalog items associated with the CatalogModifierList objects being updated.

        modifier_lists_to_enable : typing.Optional[typing.Sequence[str]]
            The IDs of the CatalogModifierList objects to enable for the CatalogItem.
            At least one of `modifier_lists_to_enable` or `modifier_lists_to_disable` must be specified.

        modifier_lists_to_disable : typing.Optional[typing.Sequence[str]]
            The IDs of the CatalogModifierList objects to disable for the CatalogItem.
            At least one of `modifier_lists_to_enable` or `modifier_lists_to_disable` must be specified.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateItemModifierListsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/catalog/update-item-modifier-lists",
            method="POST",
            json={
                "item_ids": item_ids,
                "modifier_lists_to_enable": modifier_lists_to_enable,
                "modifier_lists_to_disable": modifier_lists_to_disable,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateItemModifierListsResponse,
                    construct_type(
                        type_=UpdateItemModifierListsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update_item_taxes(
        self,
        *,
        item_ids: typing.Sequence[str],
        taxes_to_enable: typing.Optional[typing.Sequence[str]] = OMIT,
        taxes_to_disable: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateItemTaxesResponse]:
        """
        Updates the [CatalogTax](entity:CatalogTax) objects that apply to the
        targeted [CatalogItem](entity:CatalogItem) without having to perform an
        upsert on the entire item.

        Parameters
        ----------
        item_ids : typing.Sequence[str]
            IDs for the CatalogItems associated with the CatalogTax objects being updated.
            No more than 1,000 IDs may be provided.

        taxes_to_enable : typing.Optional[typing.Sequence[str]]
            IDs of the CatalogTax objects to enable.
            At least one of `taxes_to_enable` or `taxes_to_disable` must be specified.

        taxes_to_disable : typing.Optional[typing.Sequence[str]]
            IDs of the CatalogTax objects to disable.
            At least one of `taxes_to_enable` or `taxes_to_disable` must be specified.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateItemTaxesResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/catalog/update-item-taxes",
            method="POST",
            json={
                "item_ids": item_ids,
                "taxes_to_enable": taxes_to_enable,
                "taxes_to_disable": taxes_to_disable,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateItemTaxesResponse,
                    construct_type(
                        type_=UpdateItemTaxesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawCatalogClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def batch_delete(
        self, *, object_ids: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[BatchDeleteCatalogObjectsResponse]:
        """
        Deletes a set of [CatalogItem](entity:CatalogItem)s based on the
        provided list of target IDs and returns a set of successfully deleted IDs in
        the response. Deletion is a cascading event such that all children of the
        targeted object are also deleted. For example, deleting a CatalogItem will
        also delete all of its [CatalogItemVariation](entity:CatalogItemVariation)
        children.

        `BatchDeleteCatalogObjects` succeeds even if only a portion of the targeted
        IDs can be deleted. The response will only include IDs that were
        actually deleted.

        To ensure consistency, only one delete request is processed at a time per seller account.
        While one (batch or non-batch) delete request is being processed, other (batched and non-batched)
        delete requests are rejected with the `429` error code.

        Parameters
        ----------
        object_ids : typing.Sequence[str]
            The IDs of the CatalogObjects to be deleted. When an object is deleted, other objects
            in the graph that depend on that object will be deleted as well (for example, deleting a
            CatalogItem will delete its CatalogItemVariation.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BatchDeleteCatalogObjectsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/catalog/batch-delete",
            method="POST",
            json={
                "object_ids": object_ids,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BatchDeleteCatalogObjectsResponse,
                    construct_type(
                        type_=BatchDeleteCatalogObjectsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def batch_get(
        self,
        *,
        object_ids: typing.Sequence[str],
        include_related_objects: typing.Optional[bool] = OMIT,
        catalog_version: typing.Optional[int] = OMIT,
        include_deleted_objects: typing.Optional[bool] = OMIT,
        include_category_path_to_root: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[BatchGetCatalogObjectsResponse]:
        """
        Returns a set of objects based on the provided ID.
        Each [CatalogItem](entity:CatalogItem) returned in the set includes all of its
        child information including: all of its
        [CatalogItemVariation](entity:CatalogItemVariation) objects, references to
        its [CatalogModifierList](entity:CatalogModifierList) objects, and the ids of
        any [CatalogTax](entity:CatalogTax) objects that apply to it.

        Parameters
        ----------
        object_ids : typing.Sequence[str]
            The IDs of the CatalogObjects to be retrieved.

        include_related_objects : typing.Optional[bool]
            If `true`, the response will include additional objects that are related to the
            requested objects. Related objects are defined as any objects referenced by ID by the results in the `objects` field
            of the response. These objects are put in the `related_objects` field. Setting this to `true` is
            helpful when the objects are needed for immediate display to a user.
            This process only goes one level deep. Objects referenced by the related objects will not be included. For example,

            if the `objects` field of the response contains a CatalogItem, its associated
            CatalogCategory objects, CatalogTax objects, CatalogImage objects and
            CatalogModifierLists will be returned in the `related_objects` field of the
            response. If the `objects` field of the response contains a CatalogItemVariation,
            its parent CatalogItem will be returned in the `related_objects` field of
            the response.

            Default value: `false`

        catalog_version : typing.Optional[int]
            The specific version of the catalog objects to be included in the response.
            This allows you to retrieve historical versions of objects. The specified version value is matched against
            the [CatalogObject](entity:CatalogObject)s' `version` attribute. If not included, results will
            be from the current version of the catalog.

        include_deleted_objects : typing.Optional[bool]
            Indicates whether to include (`true`) or not (`false`) in the response deleted objects, namely, those with the `is_deleted` attribute set to `true`.

        include_category_path_to_root : typing.Optional[bool]
            Specifies whether or not to include the `path_to_root` list for each returned category instance. The `path_to_root` list consists
            of `CategoryPathToRootNode` objects and specifies the path that starts with the immediate parent category of the returned category
            and ends with its root category. If the returned category is a top-level category, the `path_to_root` list is empty and is not returned
            in the response payload.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BatchGetCatalogObjectsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/catalog/batch-retrieve",
            method="POST",
            json={
                "object_ids": object_ids,
                "include_related_objects": include_related_objects,
                "catalog_version": catalog_version,
                "include_deleted_objects": include_deleted_objects,
                "include_category_path_to_root": include_category_path_to_root,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BatchGetCatalogObjectsResponse,
                    construct_type(
                        type_=BatchGetCatalogObjectsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def batch_upsert(
        self,
        *,
        idempotency_key: str,
        batches: typing.Sequence[CatalogObjectBatchParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[BatchUpsertCatalogObjectsResponse]:
        """
        Creates or updates up to 10,000 target objects based on the provided
        list of objects. The target objects are grouped into batches and each batch is
        inserted/updated in an all-or-nothing manner. If an object within a batch is
        malformed in some way, or violates a database constraint, the entire batch
        containing that item will be disregarded. However, other batches in the same
        request may still succeed. Each batch may contain up to 1,000 objects, and
        batches will be processed in order as long as the total object count for the
        request (items, variations, modifier lists, discounts, and taxes) is no more
        than 10,000.

        To ensure consistency, only one update request is processed at a time per seller account.
        While one (batch or non-batch) update request is being processed, other (batched and non-batched)
        update requests are rejected with the `429` error code.

        Parameters
        ----------
        idempotency_key : str
            A value you specify that uniquely identifies this
            request among all your requests. A common way to create
            a valid idempotency key is to use a Universally unique
            identifier (UUID).

            If you're unsure whether a particular request was successful,
            you can reattempt it with the same idempotency key without
            worrying about creating duplicate objects.

            See [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) for more information.

        batches : typing.Sequence[CatalogObjectBatchParams]
            A batch of CatalogObjects to be inserted/updated atomically.
            The objects within a batch will be inserted in an all-or-nothing fashion, i.e., if an error occurs
            attempting to insert or update an object within a batch, the entire batch will be rejected. However, an error
            in one batch will not affect other batches within the same request.

            For each object, its `updated_at` field is ignored and replaced with a current [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates), and its
            `is_deleted` field must not be set to `true`.

            To modify an existing object, supply its ID. To create a new object, use an ID starting
            with `#`. These IDs may be used to create relationships between an object and attributes of
            other objects that reference it. For example, you can create a CatalogItem with
            ID `#ABC` and a CatalogItemVariation with its `item_id` attribute set to
            `#ABC` in order to associate the CatalogItemVariation with its parent
            CatalogItem.

            Any `#`-prefixed IDs are valid only within a single atomic batch, and will be replaced by server-generated IDs.

            Each batch may contain up to 1,000 objects. The total number of objects across all batches for a single request
            may not exceed 10,000. If either of these limits is violated, an error will be returned and no objects will
            be inserted or updated.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[BatchUpsertCatalogObjectsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/catalog/batch-upsert",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "batches": convert_and_respect_annotation_metadata(
                    object_=batches, annotation=typing.Sequence[CatalogObjectBatchParams], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    BatchUpsertCatalogObjectsResponse,
                    construct_type(
                        type_=BatchUpsertCatalogObjectsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def info(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[CatalogInfoResponse]:
        """
        Retrieves information about the Square Catalog API, such as batch size
        limits that can be used by the `BatchUpsertCatalogObjects` endpoint.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CatalogInfoResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/catalog/info",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CatalogInfoResponse,
                    construct_type(
                        type_=CatalogInfoResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def search(
        self,
        *,
        cursor: typing.Optional[str] = OMIT,
        object_types: typing.Optional[typing.Sequence[CatalogObjectType]] = OMIT,
        include_deleted_objects: typing.Optional[bool] = OMIT,
        include_related_objects: typing.Optional[bool] = OMIT,
        begin_time: typing.Optional[str] = OMIT,
        query: typing.Optional[CatalogQueryParams] = OMIT,
        limit: typing.Optional[int] = OMIT,
        include_category_path_to_root: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SearchCatalogObjectsResponse]:
        """
        Searches for [CatalogObject](entity:CatalogObject) of any type by matching supported search attribute values,
        excluding custom attribute values on items or item variations, against one or more of the specified query filters.

        This (`SearchCatalogObjects`) endpoint differs from the [SearchCatalogItems](api-endpoint:Catalog-SearchCatalogItems)
        endpoint in the following aspects:

        - `SearchCatalogItems` can only search for items or item variations, whereas `SearchCatalogObjects` can search for any type of catalog objects.
        - `SearchCatalogItems` supports the custom attribute query filters to return items or item variations that contain custom attribute values, where `SearchCatalogObjects` does not.
        - `SearchCatalogItems` does not support the `include_deleted_objects` filter to search for deleted items or item variations, whereas `SearchCatalogObjects` does.
        - The both endpoints have different call conventions, including the query filter formats.

        Parameters
        ----------
        cursor : typing.Optional[str]
            The pagination cursor returned in the previous response. Leave unset for an initial request.
            See [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination) for more information.

        object_types : typing.Optional[typing.Sequence[CatalogObjectType]]
            The desired set of object types to appear in the search results.

            If this is unspecified, the operation returns objects of all the top level types at the version
            of the Square API used to make the request. Object types that are nested onto other object types
            are not included in the defaults.

            At the current API version the default object types are:
            ITEM, CATEGORY, TAX, DISCOUNT, MODIFIER_LIST,
            PRICING_RULE, PRODUCT_SET, TIME_PERIOD, MEASUREMENT_UNIT,
            SUBSCRIPTION_PLAN, ITEM_OPTION, CUSTOM_ATTRIBUTE_DEFINITION, QUICK_AMOUNT_SETTINGS.

            Note that if you wish for the query to return objects belonging to nested types (i.e., COMPONENT, IMAGE,
            ITEM_OPTION_VAL, ITEM_VARIATION, or MODIFIER), you must explicitly include all the types of interest
            in this field.

        include_deleted_objects : typing.Optional[bool]
            If `true`, deleted objects will be included in the results. Defaults to `false`. Deleted objects will have their `is_deleted` field set to `true`. If `include_deleted_objects` is `true`, then the `include_category_path_to_root` request parameter must be `false`. Both properties cannot be `true` at the same time.

        include_related_objects : typing.Optional[bool]
            If `true`, the response will include additional objects that are related to the
            requested objects. Related objects are objects that are referenced by object ID by the objects
            in the response. This is helpful if the objects are being fetched for immediate display to a user.
            This process only goes one level deep. Objects referenced by the related objects will not be included.
            For example:

            If the `objects` field of the response contains a CatalogItem, its associated
            CatalogCategory objects, CatalogTax objects, CatalogImage objects and
            CatalogModifierLists will be returned in the `related_objects` field of the
            response. If the `objects` field of the response contains a CatalogItemVariation,
            its parent CatalogItem will be returned in the `related_objects` field of
            the response.

            Default value: `false`

        begin_time : typing.Optional[str]
            Return objects modified after this [timestamp](https://developer.squareup.com/docs/build-basics/working-with-dates), in RFC 3339
            format, e.g., `2016-09-04T23:59:33.123Z`. The timestamp is exclusive - objects with a
            timestamp equal to `begin_time` will not be included in the response.

        query : typing.Optional[CatalogQueryParams]
            A query to be used to filter or sort the results. If no query is specified, the entire catalog will be returned.

        limit : typing.Optional[int]
            A limit on the number of results to be returned in a single page. The limit is advisory -
            the implementation may return more or fewer results. If the supplied limit is negative, zero, or
            is higher than the maximum limit of 1,000, it will be ignored.

        include_category_path_to_root : typing.Optional[bool]
            Specifies whether or not to include the `path_to_root` list for each returned category instance. The `path_to_root` list consists of `CategoryPathToRootNode` objects and specifies the path that starts with the immediate parent category of the returned category and ends with its root category. If the returned category is a top-level category, the `path_to_root` list is empty and is not returned in the response payload. If `include_category_path_to_root` is `true`, then the `include_deleted_objects` request parameter must be `false`. Both properties cannot be `true` at the same time.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SearchCatalogObjectsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/catalog/search",
            method="POST",
            json={
                "cursor": cursor,
                "object_types": object_types,
                "include_deleted_objects": include_deleted_objects,
                "include_related_objects": include_related_objects,
                "begin_time": begin_time,
                "query": convert_and_respect_annotation_metadata(
                    object_=query, annotation=CatalogQueryParams, direction="write"
                ),
                "limit": limit,
                "include_category_path_to_root": include_category_path_to_root,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchCatalogObjectsResponse,
                    construct_type(
                        type_=SearchCatalogObjectsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def search_items(
        self,
        *,
        text_filter: typing.Optional[str] = OMIT,
        category_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        stock_levels: typing.Optional[typing.Sequence[SearchCatalogItemsRequestStockLevel]] = OMIT,
        enabled_location_ids: typing.Optional[typing.Sequence[str]] = OMIT,
        cursor: typing.Optional[str] = OMIT,
        limit: typing.Optional[int] = OMIT,
        sort_order: typing.Optional[SortOrder] = OMIT,
        product_types: typing.Optional[typing.Sequence[CatalogItemProductType]] = OMIT,
        custom_attribute_filters: typing.Optional[typing.Sequence[CustomAttributeFilterParams]] = OMIT,
        archived_state: typing.Optional[ArchivedState] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SearchCatalogItemsResponse]:
        """
        Searches for catalog items or item variations by matching supported search attribute values, including
        custom attribute values, against one or more of the specified query filters.

        This (`SearchCatalogItems`) endpoint differs from the [SearchCatalogObjects](api-endpoint:Catalog-SearchCatalogObjects)
        endpoint in the following aspects:

        - `SearchCatalogItems` can only search for items or item variations, whereas `SearchCatalogObjects` can search for any type of catalog objects.
        - `SearchCatalogItems` supports the custom attribute query filters to return items or item variations that contain custom attribute values, where `SearchCatalogObjects` does not.
        - `SearchCatalogItems` does not support the `include_deleted_objects` filter to search for deleted items or item variations, whereas `SearchCatalogObjects` does.
        - The both endpoints use different call conventions, including the query filter formats.

        Parameters
        ----------
        text_filter : typing.Optional[str]
            The text filter expression to return items or item variations containing specified text in
            the `name`, `description`, or `abbreviation` attribute value of an item, or in
            the `name`, `sku`, or `upc` attribute value of an item variation.

        category_ids : typing.Optional[typing.Sequence[str]]
            The category id query expression to return items containing the specified category IDs.

        stock_levels : typing.Optional[typing.Sequence[SearchCatalogItemsRequestStockLevel]]
            The stock-level query expression to return item variations with the specified stock levels.
            See [SearchCatalogItemsRequestStockLevel](#type-searchcatalogitemsrequeststocklevel) for possible values

        enabled_location_ids : typing.Optional[typing.Sequence[str]]
            The enabled-location query expression to return items and item variations having specified enabled locations.

        cursor : typing.Optional[str]
            The pagination token, returned in the previous response, used to fetch the next batch of pending results.

        limit : typing.Optional[int]
            The maximum number of results to return per page. The default value is 100.

        sort_order : typing.Optional[SortOrder]
            The order to sort the results by item names. The default sort order is ascending (`ASC`).
            See [SortOrder](#type-sortorder) for possible values

        product_types : typing.Optional[typing.Sequence[CatalogItemProductType]]
            The product types query expression to return items or item variations having the specified product types.

        custom_attribute_filters : typing.Optional[typing.Sequence[CustomAttributeFilterParams]]
            The customer-attribute filter to return items or item variations matching the specified
            custom attribute expressions. A maximum number of 10 custom attribute expressions are supported in
            a single call to the [SearchCatalogItems](api-endpoint:Catalog-SearchCatalogItems) endpoint.

        archived_state : typing.Optional[ArchivedState]
            The query filter to return not archived (`ARCHIVED_STATE_NOT_ARCHIVED`), archived (`ARCHIVED_STATE_ARCHIVED`), or either type (`ARCHIVED_STATE_ALL`) of items.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SearchCatalogItemsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/catalog/search-catalog-items",
            method="POST",
            json={
                "text_filter": text_filter,
                "category_ids": category_ids,
                "stock_levels": stock_levels,
                "enabled_location_ids": enabled_location_ids,
                "cursor": cursor,
                "limit": limit,
                "sort_order": sort_order,
                "product_types": product_types,
                "custom_attribute_filters": convert_and_respect_annotation_metadata(
                    object_=custom_attribute_filters,
                    annotation=typing.Sequence[CustomAttributeFilterParams],
                    direction="write",
                ),
                "archived_state": archived_state,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchCatalogItemsResponse,
                    construct_type(
                        type_=SearchCatalogItemsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update_item_modifier_lists(
        self,
        *,
        item_ids: typing.Sequence[str],
        modifier_lists_to_enable: typing.Optional[typing.Sequence[str]] = OMIT,
        modifier_lists_to_disable: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateItemModifierListsResponse]:
        """
        Updates the [CatalogModifierList](entity:CatalogModifierList) objects
        that apply to the targeted [CatalogItem](entity:CatalogItem) without having
        to perform an upsert on the entire item.

        Parameters
        ----------
        item_ids : typing.Sequence[str]
            The IDs of the catalog items associated with the CatalogModifierList objects being updated.

        modifier_lists_to_enable : typing.Optional[typing.Sequence[str]]
            The IDs of the CatalogModifierList objects to enable for the CatalogItem.
            At least one of `modifier_lists_to_enable` or `modifier_lists_to_disable` must be specified.

        modifier_lists_to_disable : typing.Optional[typing.Sequence[str]]
            The IDs of the CatalogModifierList objects to disable for the CatalogItem.
            At least one of `modifier_lists_to_enable` or `modifier_lists_to_disable` must be specified.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateItemModifierListsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/catalog/update-item-modifier-lists",
            method="POST",
            json={
                "item_ids": item_ids,
                "modifier_lists_to_enable": modifier_lists_to_enable,
                "modifier_lists_to_disable": modifier_lists_to_disable,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateItemModifierListsResponse,
                    construct_type(
                        type_=UpdateItemModifierListsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update_item_taxes(
        self,
        *,
        item_ids: typing.Sequence[str],
        taxes_to_enable: typing.Optional[typing.Sequence[str]] = OMIT,
        taxes_to_disable: typing.Optional[typing.Sequence[str]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateItemTaxesResponse]:
        """
        Updates the [CatalogTax](entity:CatalogTax) objects that apply to the
        targeted [CatalogItem](entity:CatalogItem) without having to perform an
        upsert on the entire item.

        Parameters
        ----------
        item_ids : typing.Sequence[str]
            IDs for the CatalogItems associated with the CatalogTax objects being updated.
            No more than 1,000 IDs may be provided.

        taxes_to_enable : typing.Optional[typing.Sequence[str]]
            IDs of the CatalogTax objects to enable.
            At least one of `taxes_to_enable` or `taxes_to_disable` must be specified.

        taxes_to_disable : typing.Optional[typing.Sequence[str]]
            IDs of the CatalogTax objects to disable.
            At least one of `taxes_to_enable` or `taxes_to_disable` must be specified.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateItemTaxesResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/catalog/update-item-taxes",
            method="POST",
            json={
                "item_ids": item_ids,
                "taxes_to_enable": taxes_to_enable,
                "taxes_to_disable": taxes_to_disable,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateItemTaxesResponse,
                    construct_type(
                        type_=UpdateItemTaxesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.product import Product
import typing


class SourceApplicationParams(typing_extensions.TypedDict):
    """
    Represents information about the application used to generate a change.
    """

    product: typing_extensions.NotRequired[Product]
    """
    __Read only__ The [product](entity:Product) type of the application.
    See [Product](#type-product) for possible values
    """

    application_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    __Read only__ The Square-assigned ID of the application. This field is used only if the
    [product](entity:Product) type is `EXTERNAL_API`.
    """

    name: typing_extensions.NotRequired[typing.Optional[str]]
    """
    __Read only__ The display name of the application
    (for example, `"Custom Application"` or `"Square POS 4.74 for Android"`).
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.destination_type import DestinationType


class DestinationParams(typing_extensions.TypedDict):
    """
    Information about the destination against which the payout was made.
    """

    type: typing_extensions.NotRequired[DestinationType]
    """
    Type of the destination such as a bank account or debit card.
    See [DestinationType](#type-destinationtype) for possible values
    """

    id: typing_extensions.NotRequired[str]
    """
    Square issued unique ID (also known as the instrument ID) associated with this destination.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .order_line_item_tax_type import OrderLineItemTaxType
from .money import Money
from .order_line_item_tax_scope import OrderLineItemTaxScope
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class OrderReturnTax(UncheckedBaseModel):
    """
    Represents a tax being returned that applies to one or more return line items in an order.

    Fixed-amount, order-scoped taxes are distributed across all non-zero return line item totals.
    The amount distributed to each return line item is relative to that item’s contribution to the
    order subtotal.
    """

    uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    A unique ID that identifies the returned tax only within this order.
    """

    source_tax_uid: typing.Optional[str] = pydantic.Field(default=None)
    """
    The tax `uid` from the order that contains the original tax charge.
    """

    catalog_object_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The catalog object ID referencing [CatalogTax](entity:CatalogTax).
    """

    catalog_version: typing.Optional[int] = pydantic.Field(default=None)
    """
    The version of the catalog object that this tax references.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The tax's name.
    """

    type: typing.Optional[OrderLineItemTaxType] = pydantic.Field(default=None)
    """
    Indicates the calculation method used to apply the tax.
    See [OrderLineItemTaxType](#type-orderlineitemtaxtype) for possible values
    """

    percentage: typing.Optional[str] = pydantic.Field(default=None)
    """
    The percentage of the tax, as a string representation of a decimal number.
    For example, a value of `"7.25"` corresponds to a percentage of 7.25%.
    """

    applied_money: typing.Optional[Money] = pydantic.Field(default=None)
    """
    The amount of money applied by the tax in an order.
    """

    scope: typing.Optional[OrderLineItemTaxScope] = pydantic.Field(default=None)
    """
    Indicates the level at which the `OrderReturnTax` applies. For `ORDER` scoped
    taxes, Square generates references in `applied_taxes` on all
    `OrderReturnLineItem`s. For `LINE_ITEM` scoped taxes, the tax is only applied to
    `OrderReturnLineItem`s with references in their `applied_discounts` field.
    See [OrderLineItemTaxScope](#type-orderlineitemtaxscope) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .team_member_booking_profile import TeamMemberBookingProfileParams
import typing
from .error import ErrorParams


class GetTeamMemberBookingProfileResponseParams(typing_extensions.TypedDict):
    team_member_booking_profile: typing_extensions.NotRequired[TeamMemberBookingProfileParams]
    """
    The returned team member booking profile.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class TeamMemberBookingProfileParams(typing_extensions.TypedDict):
    """
    The booking profile of a seller's team member, including the team member's ID, display name, description and whether the team member can be booked as a service provider.
    """

    team_member_id: typing_extensions.NotRequired[str]
    """
    The ID of the [TeamMember](entity:TeamMember) object for the team member associated with the booking profile.
    """

    description: typing_extensions.NotRequired[str]
    """
    The description of the team member.
    """

    display_name: typing_extensions.NotRequired[str]
    """
    The display name of the team member.
    """

    is_bookable: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Indicates whether the team member can be booked through the Bookings API or the seller's online booking channel or site (`true`) or not (`false`).
    """

    profile_image_url: typing_extensions.NotRequired[str]
    """
    The URL of the team member's image for the bookings profile.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .catalog_object import CatalogObjectParams


class CreateCatalogImageResponseParams(typing_extensions.TypedDict):
    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """

    image: typing_extensions.NotRequired[CatalogObjectParams]
    """
    The newly created `CatalogImage` including a Square-generated
    URL for the encapsulated image file.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .checkout_merchant_settings_payment_methods_afterpay_clearpay_eligibility_range import (
    CheckoutMerchantSettingsPaymentMethodsAfterpayClearpayEligibilityRangeParams,
)


class CheckoutMerchantSettingsPaymentMethodsAfterpayClearpayParams(typing_extensions.TypedDict):
    """
    The settings allowed for AfterpayClearpay.
    """

    order_eligibility_range: typing_extensions.NotRequired[
        CheckoutMerchantSettingsPaymentMethodsAfterpayClearpayEligibilityRangeParams
    ]
    """
    Afterpay is shown as an option for order totals falling within the configured range.
    """

    item_eligibility_range: typing_extensions.NotRequired[
        CheckoutMerchantSettingsPaymentMethodsAfterpayClearpayEligibilityRangeParams
    ]
    """
    Afterpay is shown as an option for item totals falling within the configured range.
    """

    enabled: typing_extensions.NotRequired[bool]
    """
    Indicates whether the payment method is enabled for the account.
    """



# This file was auto-generated by Fern from our API Definition.

from ...core.client_wrapper import SyncClientWrapper
import typing
from ...types.sort_order import SortOrder
from ...core.request_options import RequestOptions
from ...core.http_response import HttpResponse
from ...types.list_transactions_response import ListTransactionsResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...types.get_transaction_response import GetTransactionResponse
from ...types.capture_transaction_response import CaptureTransactionResponse
from ...types.void_transaction_response import VoidTransactionResponse
from ...core.client_wrapper import AsyncClientWrapper
from ...core.http_response import AsyncHttpResponse


class RawTransactionsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        location_id: str,
        *,
        begin_time: typing.Optional[str] = None,
        end_time: typing.Optional[str] = None,
        sort_order: typing.Optional[SortOrder] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ListTransactionsResponse]:
        """
        Lists transactions for a particular location.

        Transactions include payment information from sales and exchanges and refund
        information from returns and exchanges.

        Max results per [page](https://developer.squareup.com/docs/working-with-apis/pagination): 50

        Parameters
        ----------
        location_id : str
            The ID of the location to list transactions for.

        begin_time : typing.Optional[str]
            The beginning of the requested reporting period, in RFC 3339 format.

            See [Date ranges](https://developer.squareup.com/docs/build-basics/working-with-dates) for details on date inclusivity/exclusivity.

            Default value: The current time minus one year.

        end_time : typing.Optional[str]
            The end of the requested reporting period, in RFC 3339 format.

            See [Date ranges](https://developer.squareup.com/docs/build-basics/working-with-dates) for details on date inclusivity/exclusivity.

            Default value: The current time.

        sort_order : typing.Optional[SortOrder]
            The order in which results are listed in the response (`ASC` for
            oldest first, `DESC` for newest first).

            Default value: `DESC`

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for your original query.

            See [Paginating results](https://developer.squareup.com/docs/working-with-apis/pagination) for more information.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ListTransactionsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/locations/{jsonable_encoder(location_id)}/transactions",
            method="GET",
            params={
                "begin_time": begin_time,
                "end_time": end_time,
                "sort_order": sort_order,
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListTransactionsResponse,
                    construct_type(
                        type_=ListTransactionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self, location_id: str, transaction_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetTransactionResponse]:
        """
        Retrieves details for a single transaction.

        Parameters
        ----------
        location_id : str
            The ID of the transaction's associated location.

        transaction_id : str
            The ID of the transaction to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetTransactionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/locations/{jsonable_encoder(location_id)}/transactions/{jsonable_encoder(transaction_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetTransactionResponse,
                    construct_type(
                        type_=GetTransactionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def capture(
        self, location_id: str, transaction_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[CaptureTransactionResponse]:
        """
        Captures a transaction that was created with the [Charge](api-endpoint:Transactions-Charge)
        endpoint with a `delay_capture` value of `true`.


        See [Delayed capture transactions](https://developer.squareup.com/docs/payments/transactions/overview#delayed-capture)
        for more information.

        Parameters
        ----------
        location_id : str


        transaction_id : str


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CaptureTransactionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/locations/{jsonable_encoder(location_id)}/transactions/{jsonable_encoder(transaction_id)}/capture",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CaptureTransactionResponse,
                    construct_type(
                        type_=CaptureTransactionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def void(
        self, location_id: str, transaction_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[VoidTransactionResponse]:
        """
        Cancels a transaction that was created with the [Charge](api-endpoint:Transactions-Charge)
        endpoint with a `delay_capture` value of `true`.


        See [Delayed capture transactions](https://developer.squareup.com/docs/payments/transactions/overview#delayed-capture)
        for more information.

        Parameters
        ----------
        location_id : str


        transaction_id : str


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[VoidTransactionResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/locations/{jsonable_encoder(location_id)}/transactions/{jsonable_encoder(transaction_id)}/void",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    VoidTransactionResponse,
                    construct_type(
                        type_=VoidTransactionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawTransactionsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        location_id: str,
        *,
        begin_time: typing.Optional[str] = None,
        end_time: typing.Optional[str] = None,
        sort_order: typing.Optional[SortOrder] = None,
        cursor: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ListTransactionsResponse]:
        """
        Lists transactions for a particular location.

        Transactions include payment information from sales and exchanges and refund
        information from returns and exchanges.

        Max results per [page](https://developer.squareup.com/docs/working-with-apis/pagination): 50

        Parameters
        ----------
        location_id : str
            The ID of the location to list transactions for.

        begin_time : typing.Optional[str]
            The beginning of the requested reporting period, in RFC 3339 format.

            See [Date ranges](https://developer.squareup.com/docs/build-basics/working-with-dates) for details on date inclusivity/exclusivity.

            Default value: The current time minus one year.

        end_time : typing.Optional[str]
            The end of the requested reporting period, in RFC 3339 format.

            See [Date ranges](https://developer.squareup.com/docs/build-basics/working-with-dates) for details on date inclusivity/exclusivity.

            Default value: The current time.

        sort_order : typing.Optional[SortOrder]
            The order in which results are listed in the response (`ASC` for
            oldest first, `DESC` for newest first).

            Default value: `DESC`

        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint.
            Provide this to retrieve the next set of results for your original query.

            See [Paginating results](https://developer.squareup.com/docs/working-with-apis/pagination) for more information.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ListTransactionsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/locations/{jsonable_encoder(location_id)}/transactions",
            method="GET",
            params={
                "begin_time": begin_time,
                "end_time": end_time,
                "sort_order": sort_order,
                "cursor": cursor,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListTransactionsResponse,
                    construct_type(
                        type_=ListTransactionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self, location_id: str, transaction_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetTransactionResponse]:
        """
        Retrieves details for a single transaction.

        Parameters
        ----------
        location_id : str
            The ID of the transaction's associated location.

        transaction_id : str
            The ID of the transaction to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetTransactionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/locations/{jsonable_encoder(location_id)}/transactions/{jsonable_encoder(transaction_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetTransactionResponse,
                    construct_type(
                        type_=GetTransactionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def capture(
        self, location_id: str, transaction_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[CaptureTransactionResponse]:
        """
        Captures a transaction that was created with the [Charge](api-endpoint:Transactions-Charge)
        endpoint with a `delay_capture` value of `true`.


        See [Delayed capture transactions](https://developer.squareup.com/docs/payments/transactions/overview#delayed-capture)
        for more information.

        Parameters
        ----------
        location_id : str


        transaction_id : str


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CaptureTransactionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/locations/{jsonable_encoder(location_id)}/transactions/{jsonable_encoder(transaction_id)}/capture",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CaptureTransactionResponse,
                    construct_type(
                        type_=CaptureTransactionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def void(
        self, location_id: str, transaction_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[VoidTransactionResponse]:
        """
        Cancels a transaction that was created with the [Charge](api-endpoint:Transactions-Charge)
        endpoint with a `delay_capture` value of `true`.


        See [Delayed capture transactions](https://developer.squareup.com/docs/payments/transactions/overview#delayed-capture)
        for more information.

        Parameters
        ----------
        location_id : str


        transaction_id : str


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[VoidTransactionResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/locations/{jsonable_encoder(location_id)}/transactions/{jsonable_encoder(transaction_id)}/void",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    VoidTransactionResponse,
                    construct_type(
                        type_=VoidTransactionResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from ..requests.search_events_query import SearchEventsQueryParams
from ..core.request_options import RequestOptions
from ..core.http_response import HttpResponse
from ..types.search_events_response import SearchEventsResponse
from ..core.serialization import convert_and_respect_annotation_metadata
from ..core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..types.disable_events_response import DisableEventsResponse
from ..types.enable_events_response import EnableEventsResponse
from ..types.list_event_types_response import ListEventTypesResponse
from ..core.client_wrapper import AsyncClientWrapper
from ..core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawEventsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def search_events(
        self,
        *,
        cursor: typing.Optional[str] = OMIT,
        limit: typing.Optional[int] = OMIT,
        query: typing.Optional[SearchEventsQueryParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[SearchEventsResponse]:
        """
        Search for Square API events that occur within a 28-day timeframe.

        Parameters
        ----------
        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint. Provide this cursor to retrieve the next set of events for your original query.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        limit : typing.Optional[int]
            The maximum number of events to return in a single page. The response might contain fewer events. The default value is 100, which is also the maximum allowed value.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

            Default: 100

        query : typing.Optional[SearchEventsQueryParams]
            The filtering and sorting criteria for the search request. To retrieve additional pages using a cursor, you must use the original query.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[SearchEventsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/events",
            method="POST",
            json={
                "cursor": cursor,
                "limit": limit,
                "query": convert_and_respect_annotation_metadata(
                    object_=query, annotation=SearchEventsQueryParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchEventsResponse,
                    construct_type(
                        type_=SearchEventsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def disable_events(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DisableEventsResponse]:
        """
        Disables events to prevent them from being searchable.
        All events are disabled by default. You must enable events to make them searchable.
        Disabling events for a specific time period prevents them from being searchable, even if you re-enable them later.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DisableEventsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/events/disable",
            method="PUT",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DisableEventsResponse,
                    construct_type(
                        type_=DisableEventsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def enable_events(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[EnableEventsResponse]:
        """
        Enables events to make them searchable. Only events that occur while in the enabled state are searchable.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[EnableEventsResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/events/enable",
            method="PUT",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EnableEventsResponse,
                    construct_type(
                        type_=EnableEventsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def list_event_types(
        self, *, api_version: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ListEventTypesResponse]:
        """
        Lists all event types that you can subscribe to as webhooks or query using the Events API.

        Parameters
        ----------
        api_version : typing.Optional[str]
            The API version for which to list event types. Setting this field overrides the default version used by the application.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ListEventTypesResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/events/types",
            method="GET",
            params={
                "api_version": api_version,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListEventTypesResponse,
                    construct_type(
                        type_=ListEventTypesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawEventsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def search_events(
        self,
        *,
        cursor: typing.Optional[str] = OMIT,
        limit: typing.Optional[int] = OMIT,
        query: typing.Optional[SearchEventsQueryParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[SearchEventsResponse]:
        """
        Search for Square API events that occur within a 28-day timeframe.

        Parameters
        ----------
        cursor : typing.Optional[str]
            A pagination cursor returned by a previous call to this endpoint. Provide this cursor to retrieve the next set of events for your original query.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

        limit : typing.Optional[int]
            The maximum number of events to return in a single page. The response might contain fewer events. The default value is 100, which is also the maximum allowed value.

            For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).

            Default: 100

        query : typing.Optional[SearchEventsQueryParams]
            The filtering and sorting criteria for the search request. To retrieve additional pages using a cursor, you must use the original query.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[SearchEventsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/events",
            method="POST",
            json={
                "cursor": cursor,
                "limit": limit,
                "query": convert_and_respect_annotation_metadata(
                    object_=query, annotation=SearchEventsQueryParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    SearchEventsResponse,
                    construct_type(
                        type_=SearchEventsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def disable_events(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DisableEventsResponse]:
        """
        Disables events to prevent them from being searchable.
        All events are disabled by default. You must enable events to make them searchable.
        Disabling events for a specific time period prevents them from being searchable, even if you re-enable them later.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DisableEventsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/events/disable",
            method="PUT",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DisableEventsResponse,
                    construct_type(
                        type_=DisableEventsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def enable_events(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[EnableEventsResponse]:
        """
        Enables events to make them searchable. Only events that occur while in the enabled state are searchable.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[EnableEventsResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/events/enable",
            method="PUT",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    EnableEventsResponse,
                    construct_type(
                        type_=EnableEventsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def list_event_types(
        self, *, api_version: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ListEventTypesResponse]:
        """
        Lists all event types that you can subscribe to as webhooks or query using the Events API.

        Parameters
        ----------
        api_version : typing.Optional[str]
            The API version for which to list event types. Setting this field overrides the default version used by the application.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ListEventTypesResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/events/types",
            method="GET",
            params={
                "api_version": api_version,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListEventTypesResponse,
                    construct_type(
                        type_=ListEventTypesResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .money import MoneyParams
from ..types.refund_status import RefundStatus
from .additional_recipient import AdditionalRecipientParams


class RefundParams(typing_extensions.TypedDict):
    """
    Represents a refund processed for a Square transaction.
    """

    id: str
    """
    The refund's unique ID.
    """

    location_id: str
    """
    The ID of the refund's associated location.
    """

    transaction_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of the transaction that the refunded tender is part of.
    """

    tender_id: str
    """
    The ID of the refunded tender.
    """

    created_at: typing_extensions.NotRequired[str]
    """
    The timestamp for when the refund was created, in RFC 3339 format.
    """

    reason: str
    """
    The reason for the refund being issued.
    """

    amount_money: MoneyParams
    """
    The amount of money refunded to the buyer.
    """

    status: RefundStatus
    """
    The current status of the refund (`PENDING`, `APPROVED`, `REJECTED`,
    or `FAILED`).
    See [RefundStatus](#type-refundstatus) for possible values
    """

    processing_fee_money: typing_extensions.NotRequired[MoneyParams]
    """
    The amount of Square processing fee money refunded to the *merchant*.
    """

    additional_recipients: typing_extensions.NotRequired[typing.Optional[typing.Sequence[AdditionalRecipientParams]]]
    """
    Additional recipients (other than the merchant) receiving a portion of this refund.
    For example, fees assessed on a refund of a purchase by a third party integration.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.loyalty_program_status import LoyaltyProgramStatus
import typing
from .loyalty_program_reward_tier import LoyaltyProgramRewardTierParams
from .loyalty_program_expiration_policy import LoyaltyProgramExpirationPolicyParams
from .loyalty_program_terminology import LoyaltyProgramTerminologyParams
from .loyalty_program_accrual_rule import LoyaltyProgramAccrualRuleParams


class LoyaltyProgramParams(typing_extensions.TypedDict):
    """
    Represents a Square loyalty program. Loyalty programs define how buyers can earn points and redeem points for rewards.
    Square sellers can have only one loyalty program, which is created and managed from the Seller Dashboard.
    For more information, see [Loyalty Program Overview](https://developer.squareup.com/docs/loyalty/overview).
    """

    id: typing_extensions.NotRequired[str]
    """
    The Square-assigned ID of the loyalty program. Updates to 
    the loyalty program do not modify the identifier.
    """

    status: typing_extensions.NotRequired[LoyaltyProgramStatus]
    """
    Whether the program is currently active.
    See [LoyaltyProgramStatus](#type-loyaltyprogramstatus) for possible values
    """

    reward_tiers: typing_extensions.NotRequired[typing.Optional[typing.Sequence[LoyaltyProgramRewardTierParams]]]
    """
    The list of rewards for buyers, sorted by ascending points.
    """

    expiration_policy: typing_extensions.NotRequired[LoyaltyProgramExpirationPolicyParams]
    """
    If present, details for how points expire.
    """

    terminology: typing_extensions.NotRequired[LoyaltyProgramTerminologyParams]
    """
    A cosmetic name for the “points” currency.
    """

    location_ids: typing_extensions.NotRequired[typing.Optional[typing.Sequence[str]]]
    """
    The [locations](entity:Location) at which the program is active.
    """

    created_at: typing_extensions.NotRequired[str]
    """
    The timestamp when the program was created, in RFC 3339 format.
    """

    updated_at: typing_extensions.NotRequired[str]
    """
    The timestamp when the reward was last updated, in RFC 3339 format.
    """

    accrual_rules: typing_extensions.NotRequired[typing.Optional[typing.Sequence[LoyaltyProgramAccrualRuleParams]]]
    """
    Defines how buyers can earn loyalty points from the base loyalty program.
    To check for associated [loyalty promotions](entity:LoyaltyPromotion) that enable
    buyers to earn extra points, call [ListLoyaltyPromotions](api-endpoint:Loyalty-ListLoyaltyPromotions).
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from .subscription_action_type import SubscriptionActionType
from .phase import Phase
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SubscriptionAction(UncheckedBaseModel):
    """
    Represents an action as a pending change to a subscription.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of an action scoped to a subscription.
    """

    type: typing.Optional[SubscriptionActionType] = pydantic.Field(default=None)
    """
    The type of the action.
    See [SubscriptionActionType](#type-subscriptionactiontype) for possible values
    """

    effective_date: typing.Optional[str] = pydantic.Field(default=None)
    """
    The `YYYY-MM-DD`-formatted date when the action occurs on the subscription.
    """

    monthly_billing_anchor_date: typing.Optional[int] = pydantic.Field(default=None)
    """
    The new billing anchor day value, for a `CHANGE_BILLING_ANCHOR_DATE` action.
    """

    phases: typing.Optional[typing.List[Phase]] = pydantic.Field(default=None)
    """
    A list of Phases, to pass phase-specific information used in the swap.
    """

    new_plan_variation_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The target subscription plan variation that a subscription switches to, for a `SWAP_PLAN` action.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from ...requests.gift_card_activity import GiftCardActivityParams
from ...core.request_options import RequestOptions
from ...core.http_response import HttpResponse
from ...types.create_gift_card_activity_response import CreateGiftCardActivityResponse
from ...core.serialization import convert_and_respect_annotation_metadata
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper
from ...core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawActivitiesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        idempotency_key: str,
        gift_card_activity: GiftCardActivityParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateGiftCardActivityResponse]:
        """
        Creates a gift card activity to manage the balance or state of a [gift card](entity:GiftCard).
        For example, create an `ACTIVATE` activity to activate a gift card with an initial balance before first use.

        Parameters
        ----------
        idempotency_key : str
            A unique string that identifies the `CreateGiftCardActivity` request.

        gift_card_activity : GiftCardActivityParams
            The activity to create for the gift card. This activity must specify `gift_card_id` or `gift_card_gan` for the target
            gift card, the `location_id` where the activity occurred, and the activity `type` along with the corresponding activity details.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateGiftCardActivityResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            "v2/gift-cards/activities",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "gift_card_activity": convert_and_respect_annotation_metadata(
                    object_=gift_card_activity, annotation=GiftCardActivityParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateGiftCardActivityResponse,
                    construct_type(
                        type_=CreateGiftCardActivityResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawActivitiesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        idempotency_key: str,
        gift_card_activity: GiftCardActivityParams,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateGiftCardActivityResponse]:
        """
        Creates a gift card activity to manage the balance or state of a [gift card](entity:GiftCard).
        For example, create an `ACTIVATE` activity to activate a gift card with an initial balance before first use.

        Parameters
        ----------
        idempotency_key : str
            A unique string that identifies the `CreateGiftCardActivity` request.

        gift_card_activity : GiftCardActivityParams
            The activity to create for the gift card. This activity must specify `gift_card_id` or `gift_card_gan` for the target
            gift card, the `location_id` where the activity occurred, and the activity `type` along with the corresponding activity details.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateGiftCardActivityResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v2/gift-cards/activities",
            method="POST",
            json={
                "idempotency_key": idempotency_key,
                "gift_card_activity": convert_and_respect_annotation_metadata(
                    object_=gift_card_activity, annotation=GiftCardActivityParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateGiftCardActivityResponse,
                    construct_type(
                        type_=CreateGiftCardActivityResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .search_subscriptions_filter import SearchSubscriptionsFilter
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SearchSubscriptionsQuery(UncheckedBaseModel):
    """
    Represents a query, consisting of specified query expressions, used to search for subscriptions.
    """

    filter: typing.Optional[SearchSubscriptionsFilter] = pydantic.Field(default=None)
    """
    A list of query expressions.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.tender_square_account_details_status import TenderSquareAccountDetailsStatus


class TenderSquareAccountDetailsParams(typing_extensions.TypedDict):
    """
    Represents the details of a tender with `type` `SQUARE_ACCOUNT`.
    """

    status: typing_extensions.NotRequired[TenderSquareAccountDetailsStatus]
    """
    The Square Account payment's current state (such as `AUTHORIZED` or
    `CAPTURED`). See [TenderSquareAccountDetailsStatus](entity:TenderSquareAccountDetailsStatus)
    for possible values.
    See [Status](#type-status) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CatalogImage(UncheckedBaseModel):
    """
    An image file to use in Square catalogs. It can be associated with
    `CatalogItem`, `CatalogItemVariation`, `CatalogCategory`, and `CatalogModifierList` objects.
    Only the images on items and item variations are exposed in Dashboard.
    Only the first image on an item is displayed in Square Point of Sale (SPOS).
    Images on items and variations are displayed through Square Online Store.
    Images on other object types are for use by 3rd party application developers.
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    The internal name to identify this image in calls to the Square API.
    This is a searchable attribute for use in applicable query filters
    using the [SearchCatalogObjects](api-endpoint:Catalog-SearchCatalogObjects).
    It is not unique and should not be shown in a buyer facing context.
    """

    url: typing.Optional[str] = pydantic.Field(default=None)
    """
    The URL of this image, generated by Square after an image is uploaded
    using the [CreateCatalogImage](api-endpoint:Catalog-CreateCatalogImage) endpoint.
    To modify the image, use the UpdateCatalogImage endpoint. Do not change the URL field.
    """

    caption: typing.Optional[str] = pydantic.Field(default=None)
    """
    A caption that describes what is shown in the image. Displayed in the
    Square Online Store. This is a searchable attribute for use in applicable query filters
    using the [SearchCatalogObjects](api-endpoint:Catalog-SearchCatalogObjects).
    """

    photo_studio_order_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The immutable order ID for this image object created by the Photo Studio service in Square Online Store.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class OrderEntry(UncheckedBaseModel):
    """
    A lightweight description of an [order](entity:Order) that is returned when
    `returned_entries` is `true` on a [SearchOrdersRequest](api-endpoint:Orders-SearchOrders).
    """

    order_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the order.
    """

    version: typing.Optional[int] = pydantic.Field(default=None)
    """
    The version number, which is incremented each time an update is committed to the order.
    Orders that were not created through the API do not include a version number and
    therefore cannot be updated.
    
    [Read more about working with versions.](https://developer.squareup.com/docs/orders-api/manage-orders/update-orders)
    """

    location_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The location ID the order belongs to.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing
from .subscription_phase import SubscriptionPhaseParams
import typing_extensions


class CatalogSubscriptionPlanVariationParams(typing_extensions.TypedDict):
    """
    Describes a subscription plan variation. A subscription plan variation represents how the subscription for a product or service is sold.
    For more information, see [Subscription Plans and Variations](https://developer.squareup.com/docs/subscriptions-api/plans-and-variations).
    """

    name: str
    """
    The name of the plan variation.
    """

    phases: typing.Sequence[SubscriptionPhaseParams]
    """
    A list containing each [SubscriptionPhase](entity:SubscriptionPhase) for this plan variation.
    """

    subscription_plan_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The id of the subscription plan, if there is one.
    """

    monthly_billing_anchor_date: typing_extensions.NotRequired[typing.Optional[int]]
    """
    The day of the month the billing period starts.
    """

    can_prorate: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Whether bills for this plan variation can be split for proration.
    """

    successor_plan_variation_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The ID of a "successor" plan variation to this one. If the field is set, and this object is disabled at all
    locations, it indicates that this variation is deprecated and the object identified by the successor ID be used in
    its stead.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing


class TerminalRefundQuerySortParams(typing_extensions.TypedDict):
    sort_order: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The order in which results are listed.
    - `ASC` - Oldest to newest.
    - `DESC` - Newest to oldest (default).
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .customer_sort_field import CustomerSortField
import pydantic
from .sort_order import SortOrder
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CustomerSort(UncheckedBaseModel):
    """
    Represents the sorting criteria in a [search query](entity:CustomerQuery) that defines how to sort
    customer profiles returned in [SearchCustomers](api-endpoint:Customers-SearchCustomers) results.
    """

    field: typing.Optional[CustomerSortField] = pydantic.Field(default=None)
    """
    Indicates the fields to use as the sort key, which is either the default set of fields or `created_at`.
    
    The default value is `DEFAULT`.
    See [CustomerSortField](#type-customersortfield) for possible values
    """

    order: typing.Optional[SortOrder] = pydantic.Field(default=None)
    """
    Indicates the order in which results should be sorted based on the
    sort field value. Strings use standard alphabetic comparison
    to determine order. Strings representing numbers are sorted as strings.
    
    The default value is `ASC`.
    See [SortOrder](#type-sortorder) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from .device_attributes import DeviceAttributesParams
import typing
from .component import ComponentParams
from .device_status import DeviceStatusParams


class DeviceParams(typing_extensions.TypedDict):
    id: typing_extensions.NotRequired[str]
    """
    A synthetic identifier for the device. The identifier includes a standardized prefix and
    is otherwise an opaque id generated from key device fields.
    """

    attributes: DeviceAttributesParams
    """
    A collection of DeviceAttributes representing the device.
    """

    components: typing_extensions.NotRequired[typing.Optional[typing.Sequence[ComponentParams]]]
    """
    A list of components applicable to the device.
    """

    status: typing_extensions.NotRequired[DeviceStatusParams]
    """
    The current status of the device.
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from .raw_client import RawProgramsClient
from .promotions.client import PromotionsClient
from ...core.request_options import RequestOptions
from ...types.list_loyalty_programs_response import ListLoyaltyProgramsResponse
from ...types.get_loyalty_program_response import GetLoyaltyProgramResponse
from ...requests.money import MoneyParams
from ...types.calculate_loyalty_points_response import CalculateLoyaltyPointsResponse
from ...core.client_wrapper import AsyncClientWrapper
from .raw_client import AsyncRawProgramsClient
from .promotions.client import AsyncPromotionsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ProgramsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawProgramsClient(client_wrapper=client_wrapper)
        self.promotions = PromotionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawProgramsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawProgramsClient
        """
        return self._raw_client

    def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> ListLoyaltyProgramsResponse:
        """
        Returns a list of loyalty programs in the seller's account.
        Loyalty programs define how buyers can earn points and redeem points for rewards. Square sellers can have only one loyalty program, which is created and managed from the Seller Dashboard. For more information, see [Loyalty Program Overview](https://developer.squareup.com/docs/loyalty/overview).


        Replaced with [RetrieveLoyaltyProgram](api-endpoint:Loyalty-RetrieveLoyaltyProgram) when used with the keyword `main`.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListLoyaltyProgramsResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.loyalty.programs.list()
        """
        response = self._raw_client.list(request_options=request_options)
        return response.data

    def get(
        self, program_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetLoyaltyProgramResponse:
        """
        Retrieves the loyalty program in a seller's account, specified by the program ID or the keyword `main`.

        Loyalty programs define how buyers can earn points and redeem points for rewards. Square sellers can have only one loyalty program, which is created and managed from the Seller Dashboard. For more information, see [Loyalty Program Overview](https://developer.squareup.com/docs/loyalty/overview).

        Parameters
        ----------
        program_id : str
            The ID of the loyalty program or the keyword `main`. Either value can be used to retrieve the single loyalty program that belongs to the seller.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetLoyaltyProgramResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.loyalty.programs.get(
            program_id="program_id",
        )
        """
        response = self._raw_client.get(program_id, request_options=request_options)
        return response.data

    def calculate(
        self,
        program_id: str,
        *,
        order_id: typing.Optional[str] = OMIT,
        transaction_amount_money: typing.Optional[MoneyParams] = OMIT,
        loyalty_account_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CalculateLoyaltyPointsResponse:
        """
        Calculates the number of points a buyer can earn from a purchase. Applications might call this endpoint
        to display the points to the buyer.

        - If you are using the Orders API to manage orders, provide the `order_id` and (optional) `loyalty_account_id`.
        Square reads the order to compute the points earned from the base loyalty program and an associated
        [loyalty promotion](entity:LoyaltyPromotion).

        - If you are not using the Orders API to manage orders, provide `transaction_amount_money` with the
        purchase amount. Square uses this amount to calculate the points earned from the base loyalty program,
        but not points earned from a loyalty promotion. For spend-based and visit-based programs, the `tax_mode`
        setting of the accrual rule indicates how taxes should be treated for loyalty points accrual.
        If the purchase qualifies for program points, call
        [ListLoyaltyPromotions](api-endpoint:Loyalty-ListLoyaltyPromotions) and perform a client-side computation
        to calculate whether the purchase also qualifies for promotion points. For more information, see
        [Calculating promotion points](https://developer.squareup.com/docs/loyalty-api/loyalty-promotions#calculate-promotion-points).

        Parameters
        ----------
        program_id : str
            The ID of the [loyalty program](entity:LoyaltyProgram), which defines the rules for accruing points.

        order_id : typing.Optional[str]
            The [order](entity:Order) ID for which to calculate the points.
            Specify this field if your application uses the Orders API to process orders.
            Otherwise, specify the `transaction_amount_money`.

        transaction_amount_money : typing.Optional[MoneyParams]
            The purchase amount for which to calculate the points.
            Specify this field if your application does not use the Orders API to process orders.
            Otherwise, specify the `order_id`.

        loyalty_account_id : typing.Optional[str]
            The ID of the target [loyalty account](entity:LoyaltyAccount). Optionally specify this field
            if your application uses the Orders API to process orders.

            If specified, the `promotion_points` field in the response shows the number of points the buyer would
            earn from the purchase. In this case, Square uses the account ID to determine whether the promotion's
            `trigger_limit` (the maximum number of times that a buyer can trigger the promotion) has been reached.
            If not specified, the `promotion_points` field shows the number of points the purchase qualifies
            for regardless of the trigger limit.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CalculateLoyaltyPointsResponse
            Success

        Examples
        --------
        from square import Square

        client = Square(
            token="YOUR_TOKEN",
        )
        client.loyalty.programs.calculate(
            program_id="program_id",
            order_id="RFZfrdtm3mhO1oGzf5Cx7fEMsmGZY",
            loyalty_account_id="79b807d2-d786-46a9-933b-918028d7a8c5",
        )
        """
        response = self._raw_client.calculate(
            program_id,
            order_id=order_id,
            transaction_amount_money=transaction_amount_money,
            loyalty_account_id=loyalty_account_id,
            request_options=request_options,
        )
        return response.data


class AsyncProgramsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawProgramsClient(client_wrapper=client_wrapper)
        self.promotions = AsyncPromotionsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawProgramsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawProgramsClient
        """
        return self._raw_client

    async def list(self, *, request_options: typing.Optional[RequestOptions] = None) -> ListLoyaltyProgramsResponse:
        """
        Returns a list of loyalty programs in the seller's account.
        Loyalty programs define how buyers can earn points and redeem points for rewards. Square sellers can have only one loyalty program, which is created and managed from the Seller Dashboard. For more information, see [Loyalty Program Overview](https://developer.squareup.com/docs/loyalty/overview).


        Replaced with [RetrieveLoyaltyProgram](api-endpoint:Loyalty-RetrieveLoyaltyProgram) when used with the keyword `main`.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ListLoyaltyProgramsResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.loyalty.programs.list()


        asyncio.run(main())
        """
        response = await self._raw_client.list(request_options=request_options)
        return response.data

    async def get(
        self, program_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetLoyaltyProgramResponse:
        """
        Retrieves the loyalty program in a seller's account, specified by the program ID or the keyword `main`.

        Loyalty programs define how buyers can earn points and redeem points for rewards. Square sellers can have only one loyalty program, which is created and managed from the Seller Dashboard. For more information, see [Loyalty Program Overview](https://developer.squareup.com/docs/loyalty/overview).

        Parameters
        ----------
        program_id : str
            The ID of the loyalty program or the keyword `main`. Either value can be used to retrieve the single loyalty program that belongs to the seller.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetLoyaltyProgramResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.loyalty.programs.get(
                program_id="program_id",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.get(program_id, request_options=request_options)
        return response.data

    async def calculate(
        self,
        program_id: str,
        *,
        order_id: typing.Optional[str] = OMIT,
        transaction_amount_money: typing.Optional[MoneyParams] = OMIT,
        loyalty_account_id: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CalculateLoyaltyPointsResponse:
        """
        Calculates the number of points a buyer can earn from a purchase. Applications might call this endpoint
        to display the points to the buyer.

        - If you are using the Orders API to manage orders, provide the `order_id` and (optional) `loyalty_account_id`.
        Square reads the order to compute the points earned from the base loyalty program and an associated
        [loyalty promotion](entity:LoyaltyPromotion).

        - If you are not using the Orders API to manage orders, provide `transaction_amount_money` with the
        purchase amount. Square uses this amount to calculate the points earned from the base loyalty program,
        but not points earned from a loyalty promotion. For spend-based and visit-based programs, the `tax_mode`
        setting of the accrual rule indicates how taxes should be treated for loyalty points accrual.
        If the purchase qualifies for program points, call
        [ListLoyaltyPromotions](api-endpoint:Loyalty-ListLoyaltyPromotions) and perform a client-side computation
        to calculate whether the purchase also qualifies for promotion points. For more information, see
        [Calculating promotion points](https://developer.squareup.com/docs/loyalty-api/loyalty-promotions#calculate-promotion-points).

        Parameters
        ----------
        program_id : str
            The ID of the [loyalty program](entity:LoyaltyProgram), which defines the rules for accruing points.

        order_id : typing.Optional[str]
            The [order](entity:Order) ID for which to calculate the points.
            Specify this field if your application uses the Orders API to process orders.
            Otherwise, specify the `transaction_amount_money`.

        transaction_amount_money : typing.Optional[MoneyParams]
            The purchase amount for which to calculate the points.
            Specify this field if your application does not use the Orders API to process orders.
            Otherwise, specify the `order_id`.

        loyalty_account_id : typing.Optional[str]
            The ID of the target [loyalty account](entity:LoyaltyAccount). Optionally specify this field
            if your application uses the Orders API to process orders.

            If specified, the `promotion_points` field in the response shows the number of points the buyer would
            earn from the purchase. In this case, Square uses the account ID to determine whether the promotion's
            `trigger_limit` (the maximum number of times that a buyer can trigger the promotion) has been reached.
            If not specified, the `promotion_points` field shows the number of points the purchase qualifies
            for regardless of the trigger limit.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CalculateLoyaltyPointsResponse
            Success

        Examples
        --------
        import asyncio

        from square import AsyncSquare

        client = AsyncSquare(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.loyalty.programs.calculate(
                program_id="program_id",
                order_id="RFZfrdtm3mhO1oGzf5Cx7fEMsmGZY",
                loyalty_account_id="79b807d2-d786-46a9-933b-918028d7a8c5",
            )


        asyncio.run(main())
        """
        response = await self._raw_client.calculate(
            program_id,
            order_id=order_id,
            transaction_amount_money=transaction_amount_money,
            loyalty_account_id=loyalty_account_id,
            request_options=request_options,
        )
        return response.data



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from ...requests.address import AddressParams
from ...core.request_options import RequestOptions
from ...core.http_response import HttpResponse
from ...types.create_customer_card_response import CreateCustomerCardResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...core.serialization import convert_and_respect_annotation_metadata
from ...core.unchecked_base_model import construct_type
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...types.delete_customer_card_response import DeleteCustomerCardResponse
from ...core.client_wrapper import AsyncClientWrapper
from ...core.http_response import AsyncHttpResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawCardsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        customer_id: str,
        *,
        card_nonce: str,
        billing_address: typing.Optional[AddressParams] = OMIT,
        cardholder_name: typing.Optional[str] = OMIT,
        verification_token: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateCustomerCardResponse]:
        """
        Adds a card on file to an existing customer.

        As with charges, calls to `CreateCustomerCard` are idempotent. Multiple
        calls with the same card nonce return the same card record that was created
        with the provided nonce during the _first_ call.

        Parameters
        ----------
        customer_id : str
            The Square ID of the customer profile the card is linked to.

        card_nonce : str
            A card nonce representing the credit card to link to the customer.

            Card nonces are generated by the Square payment form when customers enter
            their card information. For more information, see
            [Walkthrough: Integrate Square Payments in a Website](https://developer.squareup.com/docs/web-payments/take-card-payment).

            __NOTE:__ Card nonces generated by digital wallets (such as Apple Pay)
            cannot be used to create a customer card.

        billing_address : typing.Optional[AddressParams]
            Address information for the card on file.

            __NOTE:__ If a billing address is provided in the request, the
            `CreateCustomerCardRequest.billing_address.postal_code` must match
            the postal code encoded in the card nonce.

        cardholder_name : typing.Optional[str]
            The full name printed on the credit card.

        verification_token : typing.Optional[str]
            An identifying token generated by [Payments.verifyBuyer()](https://developer.squareup.com/reference/sdks/web/payments/objects/Payments#Payments.verifyBuyer).
            Verification tokens encapsulate customer device information and 3-D Secure
            challenge results to indicate that Square has verified the buyer identity.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateCustomerCardResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/customers/{jsonable_encoder(customer_id)}/cards",
            method="POST",
            json={
                "card_nonce": card_nonce,
                "billing_address": convert_and_respect_annotation_metadata(
                    object_=billing_address, annotation=AddressParams, direction="write"
                ),
                "cardholder_name": cardholder_name,
                "verification_token": verification_token,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateCustomerCardResponse,
                    construct_type(
                        type_=CreateCustomerCardResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self, customer_id: str, card_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[DeleteCustomerCardResponse]:
        """
        Removes a card on file from a customer.

        Parameters
        ----------
        customer_id : str
            The ID of the customer that the card on file belongs to.

        card_id : str
            The ID of the card on file to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[DeleteCustomerCardResponse]
            Success
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v2/customers/{jsonable_encoder(customer_id)}/cards/{jsonable_encoder(card_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteCustomerCardResponse,
                    construct_type(
                        type_=DeleteCustomerCardResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncRawCardsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        customer_id: str,
        *,
        card_nonce: str,
        billing_address: typing.Optional[AddressParams] = OMIT,
        cardholder_name: typing.Optional[str] = OMIT,
        verification_token: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateCustomerCardResponse]:
        """
        Adds a card on file to an existing customer.

        As with charges, calls to `CreateCustomerCard` are idempotent. Multiple
        calls with the same card nonce return the same card record that was created
        with the provided nonce during the _first_ call.

        Parameters
        ----------
        customer_id : str
            The Square ID of the customer profile the card is linked to.

        card_nonce : str
            A card nonce representing the credit card to link to the customer.

            Card nonces are generated by the Square payment form when customers enter
            their card information. For more information, see
            [Walkthrough: Integrate Square Payments in a Website](https://developer.squareup.com/docs/web-payments/take-card-payment).

            __NOTE:__ Card nonces generated by digital wallets (such as Apple Pay)
            cannot be used to create a customer card.

        billing_address : typing.Optional[AddressParams]
            Address information for the card on file.

            __NOTE:__ If a billing address is provided in the request, the
            `CreateCustomerCardRequest.billing_address.postal_code` must match
            the postal code encoded in the card nonce.

        cardholder_name : typing.Optional[str]
            The full name printed on the credit card.

        verification_token : typing.Optional[str]
            An identifying token generated by [Payments.verifyBuyer()](https://developer.squareup.com/reference/sdks/web/payments/objects/Payments#Payments.verifyBuyer).
            Verification tokens encapsulate customer device information and 3-D Secure
            challenge results to indicate that Square has verified the buyer identity.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateCustomerCardResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/customers/{jsonable_encoder(customer_id)}/cards",
            method="POST",
            json={
                "card_nonce": card_nonce,
                "billing_address": convert_and_respect_annotation_metadata(
                    object_=billing_address, annotation=AddressParams, direction="write"
                ),
                "cardholder_name": cardholder_name,
                "verification_token": verification_token,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateCustomerCardResponse,
                    construct_type(
                        type_=CreateCustomerCardResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self, customer_id: str, card_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[DeleteCustomerCardResponse]:
        """
        Removes a card on file from a customer.

        Parameters
        ----------
        customer_id : str
            The ID of the customer that the card on file belongs to.

        card_id : str
            The ID of the card on file to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[DeleteCustomerCardResponse]
            Success
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v2/customers/{jsonable_encoder(customer_id)}/cards/{jsonable_encoder(card_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    DeleteCustomerCardResponse,
                    construct_type(
                        type_=DeleteCustomerCardResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .error import ErrorParams
from .webhook_subscription import WebhookSubscriptionParams


class ListWebhookSubscriptionsResponseParams(typing_extensions.TypedDict):
    """
    Defines the fields that are included in the response body of
    a request to the [ListWebhookSubscriptions](api-endpoint:WebhookSubscriptions-ListWebhookSubscriptions) endpoint.

    Note: if there are errors processing the request, the subscriptions field will not be
    present.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Information on errors encountered during the request.
    """

    subscriptions: typing_extensions.NotRequired[typing.Sequence[WebhookSubscriptionParams]]
    """
    The requested list of [Subscription](entity:WebhookSubscription)s.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    The pagination cursor to be used in a subsequent request. If empty,
    this is the final response.
    
    For more information, see [Pagination](https://developer.squareup.com/docs/build-basics/common-api-patterns/pagination).
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class BreakType(UncheckedBaseModel):
    """
    A template for a type of [break](entity:Break) that can be added to a
    [timecard](entity:Timecard), including the expected duration and paid status.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The UUID for this object.
    """

    location_id: str = pydantic.Field()
    """
    The ID of the business location this type of break applies to.
    """

    break_name: str = pydantic.Field()
    """
    A human-readable name for this type of break. The name is displayed to
    team members in Square products.
    """

    expected_duration: str = pydantic.Field()
    """
    Format: RFC-3339 P[n]Y[n]M[n]DT[n]H[n]M[n]S. The expected length of
    this break. Precision less than minutes is truncated.
    
    Example for break expected duration of 15 minutes: PT15M
    """

    is_paid: bool = pydantic.Field()
    """
    Whether this break counts towards time worked for compensation
    purposes.
    """

    version: typing.Optional[int] = pydantic.Field(default=None)
    """
    Used for resolving concurrency issues. The request fails if the version
    provided does not match the server version at the time of the request. If a value is not
    provided, Square's servers execute a "blind" write; potentially
    overwriting another writer's data.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    A read-only timestamp in RFC 3339 format.
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    A read-only timestamp in RFC 3339 format.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.booking_creator_details_creator_type import BookingCreatorDetailsCreatorType


class BookingCreatorDetailsParams(typing_extensions.TypedDict):
    """
    Information about a booking creator.
    """

    creator_type: typing_extensions.NotRequired[BookingCreatorDetailsCreatorType]
    """
    The seller-accessible type of the creator of the booking.
    See [BookingCreatorDetailsCreatorType](#type-bookingcreatordetailscreatortype) for possible values
    """

    team_member_id: typing_extensions.NotRequired[str]
    """
    The ID of the team member who created the booking, when the booking creator is of the `TEAM_MEMBER` type.
    Access to this field requires seller-level permissions.
    """

    customer_id: typing_extensions.NotRequired[str]
    """
    The ID of the customer who created the booking, when the booking creator is of the `CUSTOMER` type.
    Access to this field requires seller-level permissions.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .afterpay_details import AfterpayDetailsParams
from .clearpay_details import ClearpayDetailsParams


class BuyNowPayLaterDetailsParams(typing_extensions.TypedDict):
    """
    Additional details about a Buy Now Pay Later payment type.
    """

    brand: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The brand used for the Buy Now Pay Later payment.
    The brand can be `AFTERPAY`, `CLEARPAY` or `UNKNOWN`.
    """

    afterpay_details: typing_extensions.NotRequired[AfterpayDetailsParams]
    """
    Details about an Afterpay payment. These details are only populated if the `brand` is
    `AFTERPAY`.
    """

    clearpay_details: typing_extensions.NotRequired[ClearpayDetailsParams]
    """
    Details about a Clearpay payment. These details are only populated if the `brand` is
    `CLEARPAY`.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import pydantic
import typing
from .inventory_change import InventoryChange
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class BatchChangeInventoryRequest(UncheckedBaseModel):
    idempotency_key: str = pydantic.Field()
    """
    A client-supplied, universally unique identifier (UUID) for the
    request.
    
    See [Idempotency](https://developer.squareup.com/docs/build-basics/common-api-patterns/idempotency) in the
    [API Development 101](https://developer.squareup.com/docs/buildbasics) section for more
    information.
    """

    changes: typing.Optional[typing.List[InventoryChange]] = pydantic.Field(default=None)
    """
    The set of physical counts and inventory adjustments to be made.
    Changes are applied based on the client-supplied timestamp and may be sent
    out of order.
    """

    ignore_unchanged_counts: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates whether the current physical count should be ignored if
    the quantity is unchanged since the last physical count. Default: `true`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
from ..types.tender_buy_now_pay_later_details_brand import TenderBuyNowPayLaterDetailsBrand
from ..types.tender_buy_now_pay_later_details_status import TenderBuyNowPayLaterDetailsStatus


class TenderBuyNowPayLaterDetailsParams(typing_extensions.TypedDict):
    """
    Represents the details of a tender with `type` `BUY_NOW_PAY_LATER`.
    """

    buy_now_pay_later_brand: typing_extensions.NotRequired[TenderBuyNowPayLaterDetailsBrand]
    """
    The Buy Now Pay Later brand.
    See [Brand](#type-brand) for possible values
    """

    status: typing_extensions.NotRequired[TenderBuyNowPayLaterDetailsStatus]
    """
    The buy now pay later payment's current state (such as `AUTHORIZED` or
    `CAPTURED`). See [TenderBuyNowPayLaterDetailsStatus](entity:TenderBuyNowPayLaterDetailsStatus)
    for possible values.
    See [Status](#type-status) for possible values
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
from .fulfillment_type import FulfillmentType
import pydantic
from .fulfillment_state import FulfillmentState
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class SearchOrdersFulfillmentFilter(UncheckedBaseModel):
    """
    Filter based on [order fulfillment](entity:Fulfillment) information.
    """

    fulfillment_types: typing.Optional[typing.List[FulfillmentType]] = pydantic.Field(default=None)
    """
    A list of [fulfillment types](entity:FulfillmentType) to filter
    for. The list returns orders if any of its fulfillments match any of the fulfillment types
    listed in this field.
    See [FulfillmentType](#type-fulfillmenttype) for possible values
    """

    fulfillment_states: typing.Optional[typing.List[FulfillmentState]] = pydantic.Field(default=None)
    """
    A list of [fulfillment states](entity:FulfillmentState) to filter
    for. The list returns orders if any of its fulfillments match any of the
    fulfillment states listed in this field.
    See [FulfillmentState](#type-fulfillmentstate) for possible values
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .money import MoneyParams


class EmployeeWageParams(typing_extensions.TypedDict):
    """
    The hourly wage rate that an employee earns on a `Shift` for doing the job specified by the `title` property of this object. Deprecated at version 2020-08-26. Use [TeamMemberWage](entity:TeamMemberWage).
    """

    id: typing_extensions.NotRequired[str]
    """
    The UUID for this object.
    """

    employee_id: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The `Employee` that this wage is assigned to.
    """

    title: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The job title that this wage relates to.
    """

    hourly_rate: typing_extensions.NotRequired[MoneyParams]
    """
    Can be a custom-set hourly wage or the calculated effective hourly
    wage based on the annual wage and hours worked per week.
    """



# This file was auto-generated by Fern from our API Definition.

from ..core.unchecked_base_model import UncheckedBaseModel
import typing
import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2


class CatalogCustomAttributeDefinitionNumberConfig(UncheckedBaseModel):
    precision: typing.Optional[int] = pydantic.Field(default=None)
    """
    An integer between 0 and 5 that represents the maximum number of
    positions allowed after the decimal in number custom attribute values
    For example:
    
    - if the precision is 0, the quantity can be 1, 2, 3, etc.
    - if the precision is 1, the quantity can be 0.1, 0.2, etc.
    - if the precision is 2, the quantity can be 0.01, 0.12, etc.
    
    Default: 5
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .workweek_config import WorkweekConfigParams
from .error import ErrorParams


class ListWorkweekConfigsResponseParams(typing_extensions.TypedDict):
    """
    The response to a request for a set of `WorkweekConfig` objects. The response contains
    the requested `WorkweekConfig` objects and might contain a set of `Error` objects if
    the request resulted in errors.
    """

    workweek_configs: typing_extensions.NotRequired[typing.Sequence[WorkweekConfigParams]]
    """
    A page of `WorkweekConfig` results.
    """

    cursor: typing_extensions.NotRequired[str]
    """
    The value supplied in the subsequent request to fetch the next page of
    `WorkweekConfig` results.
    """

    errors: typing_extensions.NotRequired[typing.Sequence[ErrorParams]]
    """
    Any errors that occurred during the request.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
import typing_extensions
import typing
from .money import MoneyParams


class ProcessingFeeParams(typing_extensions.TypedDict):
    """
    Represents the Square processing fee.
    """

    effective_at: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The timestamp of when the fee takes effect, in RFC 3339 format.
    """

    type: typing_extensions.NotRequired[typing.Optional[str]]
    """
    The type of fee assessed or adjusted. The fee type can be `INITIAL` or `ADJUSTMENT`.
    """

    amount_money: typing_extensions.NotRequired[MoneyParams]
    """
    The fee amount, which might be negative, that is assessed or adjusted by Square.
    
    Positive values represent funds being assessed, while negative values represent
    funds being returned.
    """
